/***********************************************************************************************************************
 *
 * SolidBlue - open source safe data
 * Copyright (C) 2011-2012 by Tidalwave s.a.s. (http://tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://solidblue.tidalwave.it
 * SCM: https://bitbucket.org/tidalwave/solidblue-src
 *
 **********************************************************************************************************************/
package it.tidalwave.actor.io.filescan;

import javax.annotation.Nonnull;
import javax.annotation.concurrent.Immutable;
import java.io.File;
import java.nio.file.DirectoryStream.Filter;
import java.nio.file.Path;
import it.tidalwave.actor.MessageSupport;
import it.tidalwave.actor.annotation.Message;
import lombok.EqualsAndHashCode;
import lombok.Getter;
import lombok.RequiredArgsConstructor;
import lombok.ToString;
import lombok.experimental.Wither;
import static lombok.AccessLevel.PRIVATE;

/***********************************************************************************************************************
 * 
 * @stereotype Message
 * 
 * A message for requesting the scan of a folder.
 * 
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
@Message @Immutable 
@RequiredArgsConstructor(access=PRIVATE) @EqualsAndHashCode(callSuper=false) @ToString(callSuper=false)
public class FileScanRequestMessage extends MessageSupport
  {
    public static final Filter<Path> ANY = new Filter<Path>() 
      {
        @Override
        public boolean accept (final @Nonnull Path path) 
          {
            return true;
          }
      };
      
    @Nonnull @Getter
    private final Path folder;  
    
    @Nonnull @Wither @Getter
    private final Filter<Path> filter;  
    
    @Nonnull
    public static FileScanRequestMessage forFolder (final @Nonnull String path)
      {
        return forFolder(new File(path)); 
      }
    
    @Nonnull
    public static FileScanRequestMessage forFolder (final @Nonnull File file)
      {
        return forFolder(file.toPath());
      }    
    
    @Nonnull
    public static FileScanRequestMessage forFolder (final @Nonnull Path path)
      {
        return new FileScanRequestMessage(path, ANY);
      }    
  }
